﻿using System;
using System.Xml.Linq;
using System.Windows.Markup;
using System.Xml;
using System.IO;

namespace IGShowcase.AirplaneSeatingChart.Models
{
	/// <summary>
	/// The root class for the plane data (amenities, seat classes and seats) that's de-serialized from XAML.
	/// </summary>
	public class PlaneData
	{
		#region Public Properties

		#region Amenities
		/// <summary>
		/// Gets or sets a value indicating whether this instance has food.
		/// </summary>
		/// <value><c>true</c> if this instance has food; otherwise, <c>false</c>.</value>
		public bool HasFood { get; set; }
		/// <summary>
		/// Gets or sets a value indicating whether this instance has audio.
		/// </summary>
		/// <value><c>true</c> if this instance has audio; otherwise, <c>false</c>.</value>
		public bool HasAudio { get; set; }
		/// <summary>
		/// Gets or sets a value indicating whether this instance has video.
		/// </summary>
		/// <value><c>true</c> if this instance has video; otherwise, <c>false</c>.</value>
		public bool HasVideo { get; set; }
		/// <summary>
		/// Gets or sets a value indicating whether this instance has power.
		/// </summary>
		/// <value><c>true</c> if this instance has power; otherwise, <c>false</c>.</value>
		public bool HasPower { get; set; }
		/// <summary>
		/// Gets or sets a value indicating whether this instance has internet.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance has internet; otherwise, <c>false</c>.
		/// </value>
		public bool HasInternet { get; set; }
		/// <summary>
		/// Gets or sets a value indicating whether this instance has infant.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance has infant; otherwise, <c>false</c>.
		/// </value>
		public bool HasInfant { get; set; }
		#endregion Amenities

		/// <summary>
		/// Gets or sets the classes.
		/// </summary>
		/// <value>The classes.</value>
		public SeatClassCollection Classes { get; set; }

		/// <summary>
		/// Gets or sets the seats.
		/// </summary>
		/// <value>The seats.</value>
		public SeatCollection Seats { get; set; }
		#endregion Public Properties

		#region Static Methods
		/// <summary>
		/// Loads an instance of PlaneData class from a XAML file with the specified URI.
		/// </summary>
		/// <returns></returns>
		public static PlaneData Load()
		{
		    var planeData = new PlaneData();

#if SILVERLIGHT 
            planeData = (PlaneData)(XamlReader.Load(Resources.PlaneData.Xaml));
#else  // WPF
            using (var sr = new StringReader(Resources.PlaneData.Xaml))
            {
                using (var xmlReader = XmlReader.Create(sr))
                {
                    planeData = (PlaneData)(XamlReader.Load(xmlReader));
                }
            }
#endif
            return planeData;
        }

		#endregion Static Methods
	}
}
