﻿//-------------------------------------------------------------------------
// <copyright file="SeatingMapView.xaml.cs" company="Infragistics">
//
// Copyright (c) 2010 Infragistics, Inc.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// </copyright>
//-------------------------------------------------------------------------

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using IGShowcase.AirplaneSeatingChart.ViewModels;
using IGExtensions.Framework.Controls;
using Infragistics.Controls.Maps;

namespace IGShowcase.AirplaneSeatingChart.Views
{
	public partial class SeatingMapView : IGExtensions.Framework.Controls.NavigationPage
	{
		#region Private Variables
		private readonly SeatingMapViewModel _vm;

		private bool _bodyLayerLoaded;
		private bool _seatsLayerLoaded;
		#endregion Private Variables

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="SeatingMapView"/> class.
		/// </summary>
		public SeatingMapView()
		{
			InitializeComponent();

			_vm = (SeatingMapViewModel)DataContext;
		}

		#endregion Constructors

		#region Private Event Handlers
        /// <summary>
        /// Gets the navigation context of the current page. 
        /// </summary>
        public override string GetNavigationContext()
        {
            var context = base.GetNavigationContext();
            //TODO: add logic for creating navigation context
            //context = "?" + "regions=[AU,NA,EU]&minValue=[1.0]";
            return context;
        }
        // Executes when the user navigates to this page.
        protected override void OnPageNavigated(NavigationEventArgs e)
        {
            base.OnPageNavigated(e);
            //TODO: add logic for parsing navigation query  
            //double value = 0.0;
            //if (this.NavigationContext.QueryString.ContainsKey("minValue"))
            //{
            //    if (double.TryParse(this.NavigationContext.QueryString["minMag"], out value)) _vm.Value = value;
            //}
        }

	    /// <summary>
		/// Maps the layer imported.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="Infragistics.Controls.Maps.MapLayerImportEventArgs"/> instance containing the event data.</param>
		private void MapLayerImported(object sender, MapLayerImportEventArgs e)
		{
			if (e.Action != MapLayerImportAction.End) return;

			var layer = (MapLayer)sender;
			
			switch (layer.Name)
			{
				case "Body_lin":
					_bodyLayerLoaded = true;
					TrySetMapWindow();
					break;
				case "Seats_pol":
					_seatsLayerLoaded = true;
					_vm.RegisterMapElements(layer.Elements);
					_vm.PropertyChanged += VmPropertyChanged;
					TrySetMapWindow();
					break;
			}
		}

		/// <summary>
		/// Vms the property changed.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/> instance containing the event data.</param>
		private void VmPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
		{
			if(e.PropertyName == "SelectedClass")
			{
				//When the selected class changes get all the seats that are not disabled and center the map on them.
				Rect bb = Rect.Empty;
				//Find a rectangle surrounding all not disabled seats.
				foreach (MapElement mapElement in xamMap.Layers["Seats_pol"].Elements)
				{
					if (!(_vm.GetMapElementViewModel(mapElement.Name).IsDisabled))
					{
						if (bb == Rect.Empty)
						{
							bb = mapElement.WorldRect;
						}
						else
						{
							bb.Union(mapElement.WorldRect);
						}
					}
				}
				if(bb != Rect.Empty)
				{
                    xamMap.WindowRect = xamMap.Layers["Seats_pol"].WorldRect;
                    //Center the map on the rectangle
					xamMap.WindowCenter = new Point(bb.X + bb.Width / 2, bb.Y + bb.Height / 2);
                 }
			}
		}

		/// <summary>
		/// Maps the element click.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="Infragistics.Controls.Maps.MapElementClickEventArgs"/> instance containing the event data.</param>
		private void MapElementClick(object sender, MapElementClickEventArgs e)
		{
			if(e.Element.Layer.Name=="Seats_pol")
			{
				//Forward the selected element to the view model.
				_vm.SelectElement(e.Element.Name);
			}
		}

		/// <summary>
		/// Maps the element hover.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="Infragistics.Controls.Maps.MapElementHoverEventArgs"/> instance containing the event data.</param>
        private void MapElementHover(object sender, MapElementHoverEventArgs e)
        {
            if (e.Element.Layer.Name == "Seats_pol")
            {
				//Forward the hovered element to the view model.
				_vm.HoverElement(e.Element.Name);

				var map = (XamMap)sender;
				var p = e.Element.ActualSymbolOrigin;

				//Gets the position of the element center relative to the map controls top left corner.
				p = map.Viewport.RootCanvas.RenderTransform.Transform(p);
				//Set the position for the tooltip on the canvas over the map control.
				PositionTooltip(p);
            }
        }

		/// <summary>
		/// Maps the element un-hover.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="Infragistics.Controls.Maps.MapElementHoverEventArgs"/> instance containing the event data.</param>
        private void MapElementUnhover(object sender, MapElementHoverEventArgs e)
        {
            if (e.Element.Layer.Name == "Seats_pol")
            {
                _vm.UnhoverElement();
            }
        }

	
		#endregion Private Event Handlers

  

		#region Private Methods
		/// <summary>
		/// Tries the set map window.
		/// </summary>
		private void TrySetMapWindow()
		{
			//Wait for both body and seats layers to be loaded and then set the World and Window rectangles of the map.
			if (!_bodyLayerLoaded || !_seatsLayerLoaded) return;

			xamMap.WorldRect = xamMap.Layers["Body_lin"].WorldRect;
			xamMap.WindowRect = xamMap.Layers["Seats_pol"].WorldRect;
		}

		/// <summary>
		/// Positions the tooltip.
		/// </summary>
		/// <param name="elementCenter">The element center.</param>
		private void PositionTooltip(Point elementCenter)
		{
			var left = (int)(elementCenter.X) + 15;
			var top = (int)(elementCenter.Y);

			//Force a measure pass because the data context has been changed, but the dispatcher thread 
			//has not been released yet to allow for a layout update.
			tooltip.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
			//Get the new size of the tooltip control after the measure pass.
			var height = (int)(tooltip.DesiredSize.Height);

			top -= height / 2;

			//Set the tooltip position on the canvas.
			Canvas.SetLeft(tooltip, left);
			Canvas.SetTop(tooltip, top);
		}
		#endregion Private Methods
	}
}
